
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'                 Function Generator Remote Control Example
'                          Application Program A
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------

' This program is written in Microsoft QuickBasic

'     This program does the following: 
'        
'     1. Clear the screen 
'     2. Open the serial communications port for communicating with the unit. 
'     3. Make sure there is a unit connected to the serial port.   
'        If we can't find one, wait until it is connected up. 
'     4. Command the unit to generate 18.432 MHz at 0.0 dBm. 
'     5. Wait for the user to press a key.
'     6. Switch the unit to DTMF Detection mode.
'     7. Wait for the unit to detect a DTMF digit and print it to
'        the terminal. 


DECLARE SUB Delay (X%)  ' Declare functions used below.
                        '  (The Delay function waits x mS then returns)
DECLARE SUB Pause ()    '  (The Pause fctn waits for a keypress then returns)

   CLS                  ' Clear the screen 

' Open serial port 1 for communication with the unit with these parameters:
'   Ignore the CD,CTS,DCD flow control lines
'   Set a 5 sec maximum timeout for basic to open the port
'   Set the receive buffer to 2K bytes
'   Disable the RTS flow control line
'   Set the transmit buffer to 1K bytes
'   File access type is Random
'   Set random access buffer size to 2K

   OPEN "COM1:9600,n,8,1,CD0,CS0,DS0,OP5000,RB2048,RS,TB1024" FOR RANDOM AS #1 LEN = 2048


'----------- Make sure a unit is attached to the serial port --------------

' If we send the unit a control E char, it will respond with a control C.  
' We use this feature to see if a unit is attached.
' If a unit cannot be found, wait until one is attached.
' User can abort the program at this point by pressing any key.


' When power is applied to the unit, it prints a menu to the serial port.  
' We wish to disregard these characters, so flush them.

   GOSUB FlushBuff1             ' Flush all received chars from buffer

' Tell the user to connect up a function generator

   PRINT "Connect the function generator to the serial port and power the unit"


ConnectLoop:
   PRINT #1, CHR$(5);           ' Print a control E to unit (ASCII char 5)

' Wait for the unit to give us some characters. When they arrive, put
'  then into the string a$

   GOSUB GetResponse1           ' Wait for and get response to a$

' Allow the user to exit the program by pressing a key.

   IF INKEY$ <> "" THEN SYSTEM  ' Exit on first terminal keypress

' Did we get a control C (ascii char 3) from the unit?

   IF INSTR(a$, CHR$(3)) THEN GOTO GotSG100

   GOTO ConnectLoop
GotSG100:
     
'----------- Set a frequency and level on the unit --------------
' Send the following command string to the unit:
' A F1 18.432Z F2 0.0Z F0
' These characters have the following meaning:
'   A      - Reset the unit to Sinewave mode
'   F1     - Move cursor to field 1 (the frequency field)
' 18.432Z  - Enter a frequency of 18.432 MHz
'   F2     - Move cursor to field 2 (the level field)
'  0.0Z    - Enter a level of 0.0 dBm
'   F0     - Move cursor to field 0 (turns cursor off)

   PRINT #1, "A F1 18.432Z F2 0.0Z F0";
     
' After the unit executes each command, it'll return a prompt character (">").
'  We wish to wait until after all prompts have come in, since that's when 
'  the unit has finished executing the last command. The GetResponse1 
'  subroutine has a timeout feature which will accomplish this purpose.

   GOSUB GetResponse1           ' Wait for and get response to a$

' Let the user know what's going on

   CLS
   PRINT "CW mode selected. Frequency and Level have been set."
   PRINT : PRINT
   PRINT "Press any key to go to DTMF Detection mode"

' Wait for the user to press a key before we switch to a new mode.
   CALL Pause

     
'----------- Switch to DTMF Detection mode --------------
' This example illustrates how to parse information from the unit

   PRINT #1, "MD";          ' Command the unit to go to DTMF Detection mode

   GOSUB PromptWait             ' Wait for unit to finish this command

   GOSUB FlushBuff1             ' Flush all received chars 


' Let the user know what's going on
   CLS
   PRINT : PRINT
   PRINT "Now in DTMF Detection mode. All DTMF detections will be printed."
   PRINT

' When the unit detects a DTMF digit, it will print the following text
'  to the terminal port: CR,LF,"DTMF: 7"

DTMFLoop:
   GOSUB GetResponse1          ' Wait for and get response to a$

' The unit prints a semicolon (:) character whenever it is reporting data.
' The data follows immediately after the semicolon.

   IF INSTR(a$, ":") THEN
     a = INSTR(a$, ":") + 1    ' Get the data that follows the ":" character
     a$ = MID$(a$, a, 2)       ' (Strip off everything else)
     PRINT "Detected DTMF digit is: "; a$
   END IF

' Allow the user to exit the program by pressing a key.

   IF INKEY$ <> "" THEN SYSTEM  ' Exit on first terminal keypress

   GOTO DTMFLoop            ' Otherwise keep looking for more DTMF detections

'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'                              SUBROUTINES
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------

'------------------------------------------------------------------------------
'               Get ascii response from the function generator
' Wait for an incoming char, then collect incoming stuff until we get a
'   800 mS Rx char timeout
' Returns with a$ = received stuff
'------------------------------------------------------------------------------
GetResponse1:

   IF LOC(1) = 0 THEN GOTO GetResponse1     ' Wait for incoming char

   a$ = ""

GetRespLoop1:

   a$ = a$ + INPUT$(LOC(1), #1)             ' Get all waiting chars in Rx buffer

' Now wait for a timeout time to make sure no new stuff is coming in.
   CALL Delay(.8)                           ' Do a 800 mS delay

   IF LOC(1) <> 0 THEN GOTO GetRespLoop1    ' If something came in during
                                            '  our timeout, grab it
   RETURN

'------------------------------------------------------------------------------
'                 Wait for a command prompt
' Returns A$ = all stuff received before the command prompt.
'------------------------------------------------------------------------------
PromptWait:

' If we have no chars in Rx buffer, wait until we have 1 or more

   a$ = ""

PromptWaitLoop:
   IF LOC(1) = 0 THEN GOTO PromptWaitLoop

   a$ = a$ + INPUT$(LOC(1), #1)    ' Get all waiting chars in Rx buffer

   IF INSTR(a$, ">") THEN GOTO GotPrompt

   GOTO PromptWaitLoop

' Here we got a prompt. Exit
GotPrompt:

   RETURN


'------------------------------------------------------------------------------
'                 Flush all Rx chars from Port 1 buffer
'------------------------------------------------------------------------------
FlushBuff1:

   IF LOC(1) = 0 THEN GOTO DoTimeout1

   a$ = INPUT$(LOC(1), #1)      ' Get all waiting chars in Rx buffer

' Now wait for a timeout time to make sure no new stuff is coming in.
DoTimeout1:
   CALL Delay(.8)               ' Do a 800 mS delay

   IF LOC(1) <> 0 THEN GOTO FlushBuff1          ' If something came in during
                                                '  our timeout, flush it
   RETURN

'------------------------------------------------------------------------------
'                             Wait for x mS
' Function to delay x number of mS.
'------------------------------------------------------------------------------
  SUB Delay (X%) STATIC

   start! = TIMER                       ' Returns no. of secs since midnight.
                                        ' Example: 65445.15

DelayLoop:                              ' Do a x mS delay
   IF TIMER < start! + X% / 1000 THEN GOTO DelayLoop

  END SUB

'------------------------------------------------------------------------------
'                          Wait for a keypress
'------------------------------------------------------------------------------
  SUB Pause STATIC

   DO
   LOOP UNTIL INKEY$ <> ""       ' Wait for keypress to continue

   END SUB

